import os

from django.conf import settings
from django.core.management import call_command
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.utils import override_settings
from django.utils import simplejson
from django.utils.translation import ugettext as _

from ..models import UploadedFile
from ..widgets import AjaxUploadException


TEST_FILEPATH = os.path.dirname(__file__) + '/files/test.png'


class UploaderTestHelper(object):

    def tearDown(self):
        # Delete all uploaded files created during testing
        for up in UploadedFile.objects.all():
            if up.file:
                up.file.delete()

    def create_uploaded_file(self, **kwargs):
        defaults = {
            'file': 'test.png'
        }
        defaults.update(kwargs)
        return UploadedFile.objects.create(**defaults)


class AjaxUploadTests(UploaderTestHelper, TestCase):

    def test_upload_file_submission_saves_file_with_different_name_and_returns_json(self):
        post_data = {
            'file': open(TEST_FILEPATH)
        }
        response = self.client.post(reverse('ajax-upload'), post_data)
        self.assertEqual(response.status_code, 200)
        json = simplejson.loads(response.content)
        uploaded_file = UploadedFile.objects.get()
        self.assertTrue(os.path.exists(uploaded_file.file.path))
        self.assertEqual(json['path'], uploaded_file.file.url)
        # This is a not-so-good test to verify that the filename name is modified
        self.assertTrue(len(os.path.basename(uploaded_file.file.name)) > 16)

    def test_upload_file_submission_missing_file_returns_error(self):
        post_data = {
            'file': ''
        }
        response = self.client.post(reverse('ajax-upload'), post_data)
        self.assertEqual(response.status_code, 400)
        json = simplejson.loads(response.content)
        self.assertTrue('errors' in json)
        self.assertEqual(json['errors']['file'][0], _('This field is required.'))

    def test_upload_file_get_request_returns_405(self):
        response = self.client.get(reverse('ajax-upload'))
        self.assertEqual(response.status_code, 405)


class AjaxFileInputTests(UploaderTestHelper, TestCase):
    urls = 'ajax_upload.tests.urls'

    def test_submit_form_with_uploaded_file_path(self):
        # First ajax upload the file to the uploader
        post_data = {
            'file': open(TEST_FILEPATH)
        }
        response = self.client.post(reverse('ajax-upload'), post_data)
        self.assertEqual(response.status_code, 200)
        json = simplejson.loads(response.content)
        uploaded_file = UploadedFile.objects.get()
        self.assertTrue(os.path.exists(uploaded_file.file.path))

        # Now submit the original form with the path of the uploaded file
        post_data = {
            'my_file': json['path'],
            'my_image': json['path']  # We're testing both AjaxFileField and AjaxImageField
        }
        response = self.client.post(reverse('ajax-uploads-test'), post_data)
        self.assertEqual(response.status_code, 200)
        parsed = simplejson.loads(response.content)
        self.assertEqual(parsed['uploaded_file_name'], json['path'].replace(settings.MEDIA_URL, ''))
        self.assertEqual(parsed['uploaded_image_name'], json['path'].replace(settings.MEDIA_URL, ''))

    def test_submit_form_with_file_path_that_doesnt_exist_returns_error(self):
        post_data = {
            'my_file': '%sinvalid-file-path.txt' % settings.MEDIA_URL,
            'my_image': '%sinvalid-file-path.png' % settings.MEDIA_URL
        }
        try:
            self.client.post(reverse('ajax-uploads-test'), post_data)
        except AjaxUploadException, err:
            self.assertTrue(str(err).startswith(_('Invalid file path:')))
        else:
            self.fail()

    def test_submit_form_with_empty_path_clears_existing_file(self):
        post_data = {
            'my_file': '',
            'my_image': ''
        }
        response = self.client.post(reverse('ajax-uploads-test'), post_data)
        self.assertEqual(response.status_code, 200)
        parsed = simplejson.loads(response.content)
        self.assertEqual(parsed['uploaded_file_name'], 'False')
        self.assertEqual(parsed['uploaded_image_name'], 'False')

    def test_submit_form_with_external_file_path_returns_error(self):
        post_data = {
            'my_file': 'http://www.google.com/invalid.txt',
            'my_image': 'http://www.google.com/invalid.png'
        }
        try:
            self.client.post(reverse('ajax-uploads-test'), post_data)
        except AjaxUploadException, err:
            self.assertTrue(str(err).startswith(_('File path not allowed:')))
        else:
            self.fail()

    def test_submit_form_normally_with_file_data_in_multipart_format(self):
        # Here we will NOT use the AJAX uploader to ensure the file field works normally.
        post_data = {
            'my_file': open(TEST_FILEPATH),
            'my_image': open(TEST_FILEPATH)
        }
        response = self.client.post(reverse('ajax-uploads-test'), post_data)
        self.assertEqual(response.status_code, 200)
        parsed = simplejson.loads(response.content)
        self.assertTrue('errors' not in parsed)
        self.assertTrue('uploaded_file_name' in parsed)
        self.assertTrue('uploaded_image_name' in parsed)



from django.http import HttpResponse, HttpResponseBadRequest
from django.utils import simplejson

from ..tests.forms import TestForm


def test_view(request):
    form = TestForm(data=request.POST, files=request.FILES)
    if form.is_valid():
        data = {
            'uploaded_file_name': str(form.cleaned_data['my_file']),
            'uploaded_image_name': str(form.cleaned_data['my_image'])
        }
        return HttpResponse(
            simplejson.dumps(data), mimetype='application/json'
        )
    else:
        return HttpResponseBadRequest(
            simplejson.dumps({'errors': form.errors}),
            mimetype='application/json'
        )



from django import forms

from ..widgets import AjaxClearableFileInput


class TestForm(forms.Form):
    my_file = forms.FileField(required=False, widget=AjaxClearableFileInput)
    my_image = forms.ImageField(required=False, widget=AjaxClearableFileInput)



from django import forms
from django.conf import settings
from django.core.files import File
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from .models import UploadedFile


class AjaxUploadException(Exception):
    pass


class AjaxClearableFileInput(forms.ClearableFileInput):
    template_with_clear = ''  # We don't need this
    template_with_initial = '%(input)s'

    def render(self, name, value, attrs=None):
        attrs = attrs or {}
        if value:
            filename = u'%s%s' % (settings.MEDIA_URL, value)
        else:
            filename = ''
        attrs.update({
            'class': attrs.get('class', '') + 'ajax-upload',
            'data-filename': filename,  # This is so the javascript can get the actual value
            'data-required': self.is_required or '',
            'data-upload-url': reverse('ajax-upload')
        })
        output = super(AjaxClearableFileInput, self).render(name, value, attrs)
        return mark_safe(output)

    def value_from_datadict(self, data, files, name):
        # If a file was uploaded or the clear checkbox was checked, use that.
        file = super(AjaxClearableFileInput, self).value_from_datadict(data, files, name)
        if file is not None:  # super class may return a file object, False, or None
            return file  # Default behaviour
        elif name in data:  # This means a file path was specified in the POST field
            file_path = data.get(name)
            if not file_path:
                return False  # False means clear the existing file
            elif file_path.startswith(settings.MEDIA_URL):
                # Strip and media url to determine the path relative to media root
                relative_path = file_path[len(settings.MEDIA_URL):]
                try:
                    uploaded_file = UploadedFile.objects.get(file=relative_path)
                except UploadedFile.DoesNotExist:
                    raise AjaxUploadException(u'%s %s' % (_('Invalid file path:'), relative_path))
                else:
                    return File(uploaded_file.file)
            else:
                raise AjaxUploadException(u'%s %s' % (_('File path not allowed:'), file_path))
        return None



from django.http import HttpResponse, HttpResponseBadRequest
from django.utils import simplejson
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

from .forms import UploadedFileForm


@csrf_exempt
@require_POST
def upload(request):
    form = UploadedFileForm(data=request.POST, files=request.FILES)
    if form.is_valid():
        uploaded_file = form.save()
        data = {
            'path': uploaded_file.file.url,
        }
        return HttpResponse(simplejson.dumps(data))
    else:
        return HttpResponseBadRequest(simplejson.dumps({'errors': form.errors}))



from distutils.core import setup

import ajax_upload


setup(
    name='django-ajax-upload-widget',
    packages=['ajax_upload', 'ajax_upload.tests'],
    version=ajax_upload.__version__,
    description=ajax_upload.__doc__,
    long_description=open('README.rst').read(),
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
    author='Zach Mathew',
    url='http://github.com/zmathew/django-ajax-upload-widget',
    license='BSD',
)
